@@ -624,8 +624,8 @@ Tk_GetReliefFromObj(
 				 * from. */
     int *resultPtr)		/* Where to place the answer. */
 {
-    return Tcl_GetIndexFromObj(interp, objPtr, reliefStrings, "relief", 0,
-	    resultPtr);
+    return Tcl_GetIndexFromObjStruct(interp, objPtr, reliefStrings,
+	    sizeof(char *), "relief", 0, resultPtr);
 }
 
 /*
