@@ -226,9 +226,6 @@ void TkSubtractRegion (TkRegion a, TkRegion b, TkRegion c)
 #   endif
 #endif /* !__WIN32__ */
 
-#define reserved274 (void (*)(void)) Tk_DefineBitmap
-#define reserved275 (void (*)(void)) Tk_GetBitmapFromData
-
 /*
  * WARNING: The contents of this file is automatically generated by the
  * tools/genStubs.tcl script. Any modifications to the function declarations
@@ -1106,8 +1103,6 @@ const TkStubs tkStubs = {
     Tk_Interp, /* 271 */
     Tk_CreateOldImageType, /* 272 */
     Tk_CreateOldPhotoImageFormat, /* 273 */
-    reserved274, /* 274 */
-    reserved275, /* 275 */
 };
 
 /* !END!: Do not edit above this line. */
