@@ -13,9 +13,10 @@
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- */ 
+ */
 
 #include "tkMacOSXPrivate.h"
+#include "tkBusy.h"
 
 /*
  * One of the following structures exists for each container in this
@@ -89,8 +90,7 @@ Tk_MacOSXSetEmbedHandler(
     Tk_MacOSXEmbedGetOffsetInParentProc *getOffsetProc)
 {
     if (tkMacOSXEmbedHandler == NULL) {
-	tkMacOSXEmbedHandler = (TkMacOSXEmbedHandler *)
-		ckalloc(sizeof(TkMacOSXEmbedHandler));
+	tkMacOSXEmbedHandler = ckalloc(sizeof(TkMacOSXEmbedHandler));
     }
     tkMacOSXEmbedHandler->registerWinProc = registerWinProc;
     tkMacOSXEmbedHandler->getPortProc = getPortProc;
@@ -134,7 +134,7 @@ TkpMakeWindow(
 	 * Allocate sub window
 	 */
 
-	macWin = (MacDrawable *) ckalloc(sizeof(MacDrawable));
+	macWin = ckalloc(sizeof(MacDrawable));
 	if (macWin == NULL) {
 	    winPtr->privatePtr = NULL;
 	    return None;
@@ -171,50 +171,6 @@ TkpMakeWindow(
     return (Window) macWin;
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TkpScanWindowId --
- *
- *	Given a string, produce the corresponding Window Id.
- *
- * Results:
- *      The return value is normally TCL_OK; in this case *idPtr will be set
- *      to the Window value equivalent to string. If string is improperly
- *      formed then TCL_ERROR is returned and an error message will be left in
- *      the interp's result.
- *
- * Side effects:
- *      None.
- *
- *----------------------------------------------------------------------
- */
-
-int
-TkpScanWindowId(
-    Tcl_Interp *interp,
-    CONST char * string,
-    Window *idPtr)
-{
-    int code;
-    Tcl_Obj obj;
-
-    obj.refCount = 1;
-    obj.bytes = (char *) string;	/* DANGER?! */
-    obj.length = strlen(string);
-    obj.typePtr = NULL;
-
-    code = Tcl_GetLongFromObj(interp, &obj, (long *)idPtr);
-
-    if (obj.refCount > 1) {
-	Tcl_Panic("invalid sharing of Tcl_Obj on C stack");
-    }
-    if (obj.typePtr && obj.typePtr->freeIntRepProc) {
-	obj.typePtr->freeIntRepProc(&obj);
-    }
-    return code;
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -252,13 +208,14 @@ TkpUseWindow(
     Container *containerPtr;
 
     if (winPtr->window != None) {
-	Tcl_AppendResult(interp, "can't modify container after widget is "
-		"created", NULL);
+	Tcl_SetObjResult(interp, Tcl_NewStringObj(
+		"can't modify container after widget is created", -1));
+	Tcl_SetErrorCode(interp, "TK", "EMBED", "POST_CREATE", NULL);
 	return TCL_ERROR;
     }
 
     /*
-     * Decode the container window ID, and look for it among the list of
+     * Decode the container pointer, and look for it among the list of
      * available containers.
      *
      * N.B. For now, we are limiting the containers to be in the same Tk
@@ -266,17 +223,17 @@ TkpUseWindow(
      * containers.
      */
 
-    if (TkpScanWindowId(interp, string, (Window *)&parent) != TCL_OK) {
+    if (Tcl_GetInt(interp, string, (int*) &parent) != TCL_OK) {
 	return TCL_ERROR;
     }
 
     usePtr = (TkWindow *) Tk_IdToWindow(winPtr->display, (Window) parent);
-    if (usePtr != NULL) {
-	if (!(usePtr->flags & TK_CONTAINER)) {
-	    Tcl_AppendResult(interp, "window \"", usePtr->pathName,
-		    "\" doesn't have -container option set", NULL);
-	    return TCL_ERROR;
-	}
+    if (usePtr != NULL && !(usePtr->flags & TK_CONTAINER)) {
+	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		"window \"%s\" doesn't have -container option set",
+		usePtr->pathName));
+	Tcl_SetErrorCode(interp, "TK", "EMBED", "CONTAINER", NULL);
+	return TCL_ERROR;
     }
 
     /*
@@ -304,7 +261,7 @@ TkpUseWindow(
      * Make the embedded window.
      */
 
-    macWin = (MacDrawable *) ckalloc(sizeof(MacDrawable));
+    macWin = ckalloc(sizeof(MacDrawable));
     if (macWin == NULL) {
 	winPtr->privatePtr = NULL;
 	return TCL_ERROR;
@@ -349,25 +306,27 @@ TkpUseWindow(
 
     if (containerPtr == NULL) {
 	/*
-	 * If someone has registered an in process embedding handler, then
+	 * If someone has registered an in-process embedding handler, then
 	 * see if it can handle this window...
 	 */
 
 	if (tkMacOSXEmbedHandler == NULL ||
 		tkMacOSXEmbedHandler->registerWinProc((long) parent,
 		(Tk_Window) winPtr) != TCL_OK) {
-	    Tcl_AppendResult(interp, "The window ID ", string,
-		    " does not correspond to a valid Tk Window.", NULL);
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "The window ID %s does not correspond to a valid Tk Window",
+		    string));
+	    Tcl_SetErrorCode(interp, "TK", "EMBED", "HANDLE", NULL);
 	    return TCL_ERROR;
-	} else {
-	    containerPtr = (Container *) ckalloc(sizeof(Container));
-
-	    containerPtr->parentPtr = NULL;
-	    containerPtr->embedded = (Window) macWin;
-	    containerPtr->embeddedPtr = macWin->winPtr;
-	    containerPtr->nextPtr = firstContainerPtr;
-	    firstContainerPtr = containerPtr;
 	}
+
+	containerPtr = ckalloc(sizeof(Container));
+
+	containerPtr->parentPtr = NULL;
+	containerPtr->embedded = (Window) macWin;
+	containerPtr->embeddedPtr = macWin->winPtr;
+	containerPtr->nextPtr = firstContainerPtr;
+	firstContainerPtr = containerPtr;
     } else {
 	/*
 	 * The window is embedded in another Tk window.
@@ -433,7 +392,7 @@ TkpMakeContainer(
      */
 
     Tk_MakeWindowExist(tkwin);
-    containerPtr = (Container *) ckalloc(sizeof(Container));
+    containerPtr = ckalloc(sizeof(Container));
     containerPtr->parent = Tk_WindowId(tkwin);
     containerPtr->parentPtr = winPtr;
     containerPtr->embedded = None;
@@ -1158,10 +1117,64 @@ EmbedWindowDeleted(
 	} else {
 	    prevPtr->nextPtr = containerPtr->nextPtr;
 	}
-	ckfree((char *) containerPtr);
+	ckfree(containerPtr);
     }
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkpShowBusyWindow, TkpHideBusyWindow, TkpMakeTransparentWindowExist,
+ * TkpCreateBusy --
+ *
+ *	Portability layer for busy windows. Holds platform-specific gunk for
+ *	the [tk busy] command, which is currently a dummy implementation for
+ *	OSX/Aqua. The individual functions are supposed to do the following:
+ *
+ * TkpShowBusyWindow --
+ *	Make the busy window appear.
+ *
+ * TkpHideBusyWindow --
+ *	Make the busy window go away.
+ *
+ * TkpMakeTransparentWindowExist --
+ *	Actually make a transparent window.
+ *
+ * TkpCreateBusy --
+ *	Creates the platform-specific part of a busy window structure.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TkpShowBusyWindow(
+    TkBusy busy)
+{
+}
+
+void
+TkpHideBusyWindow(
+    TkBusy busy)
+{
+}
+
+void
+TkpMakeTransparentWindowExist(
+    Tk_Window tkwin,		/* Token for window. */
+    Window parent)		/* Parent window. */
+{
+}
+
+void
+TkpCreateBusy(
+    Tk_FakeWin *winPtr,
+    Tk_Window tkRef,
+    Window* parentPtr,
+    Tk_Window tkParent,
+    TkBusy busy)
+{
+}
+
 /*
  * Local Variables:
  * mode: objc
