@@ -23,26 +23,26 @@ enum {
 
 @implementation TKApplication(TKEvent)
 /* TODO: replace by +[addLocalMonitorForEventsMatchingMask ? */
-- (NSEvent *)tkProcessEvent:(NSEvent *)theEvent {
+- (NSEvent *) tkProcessEvent: (NSEvent *) theEvent
+{
 #ifdef TK_MAC_DEBUG_EVENTS
     TKLog(@"-[%@(%p) %s] %@", [self class], self, _cmd, theEvent);
 #endif
     NSEvent	    *processedEvent = theEvent;
     NSEventType	    type = [theEvent type];
     NSInteger	    subtype;
+    NSUInteger	    flags;
 
     switch ((NSInteger)type) {
     case NSAppKitDefined:
         subtype = [theEvent subtype];
 
 	switch (subtype) {
-	    /* Ignored at the moment. */
 	case NSApplicationActivatedEventType:
 	    break;
 	case NSApplicationDeactivatedEventType:
 	    break;
 	case NSWindowExposedEventType:
-	    break;
 	case NSScreenChangedEventType:
 	    break;
 	case NSWindowMovedEventType:
@@ -53,12 +53,13 @@ enum {
         default:
             break;
 	}
-	break; /* AppkitEvent. Return theEvent */
+	break;
     case NSKeyUp:
     case NSKeyDown:
     case NSFlagsChanged:
+	flags = [theEvent modifierFlags];
 	processedEvent = [self tkProcessKeyEvent:theEvent];
-	break; /* Key event.  Return the processed event. */
+	break;
     case NSLeftMouseDown:
     case NSLeftMouseUp:
     case NSRightMouseDown:
@@ -75,7 +76,7 @@ enum {
     case NSTabletPoint:
     case NSTabletProximity:
 	processedEvent = [self tkProcessMouseEvent:theEvent];
-	break; /* Mouse event.  Return the processed event. */
+	break;
 #if 0
     case NSSystemDefined:
         subtype = [theEvent subtype];
@@ -99,7 +100,7 @@ enum {
 #endif
 
     default:
-	break; /* return theEvent */
+	break;
     }
     return processedEvent;
 }
@@ -112,26 +113,35 @@ enum {
  *
  * TkMacOSXFlushWindows --
  *
- *	This routine flushes all the visible windows of the application. It is
+ *	This routine flushes all the windows of the application. It is
  *	called by XSync().
  *
  * Results:
  *	None.
  *
  * Side effects:
- *	Flushes all visible Cocoa windows
+ *	Flushes all Carbon windows
  *
  *----------------------------------------------------------------------
  */
+
 MODULE_SCOPE void
 TkMacOSXFlushWindows(void)
 {
-    NSArray *macWindows = [NSApp orderedWindows];
+    NSInteger windowCount;
+    NSInteger *windowNumbers;
 
-    for (NSWindow *w in macWindows) {
-	if (TkMacOSXGetXWindow(w)) {
-	    [w flushWindow];
+    NSCountWindows(&windowCount);
+    if(windowCount) {
+	windowNumbers = ckalloc(windowCount * sizeof(NSInteger));
+	NSWindowList(windowCount, windowNumbers);
+	for (NSInteger index = 0; index < windowCount; index++) {
+	    NSWindow *w = [NSApp windowWithWindowNumber:windowNumbers[index]];
+	    if (TkMacOSXGetXWindow(w)) {
+		[w flushWindow];
+	    }
 	}
+	ckfree(windowNumbers);
     }
 }
 
