@@ -7,7 +7,6 @@
  * Copyright (c) 1995-1997 Sun Microsystems, Inc.
  * Copyright 2001-2009, Apple Inc.
  * Copyright (c) 2005-2009 Daniel A. Steffen <das@users.sourceforge.net>
- * Copyright 2015 Marc Culler. 
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -19,14 +18,14 @@
 #include <pthread.h>
 #import <objc/objc-auto.h>
 
-/* This is not used for anything at the moment. */
 typedef struct ThreadSpecificData {
-    int initialized;
+    int initialized, sendEventNestingLevel;
+    NSEvent *currentEvent;
 } ThreadSpecificData;
 static Tcl_ThreadDataKey dataKey;
 
-#define TSD_INIT() ThreadSpecificData *tsdPtr = Tcl_GetThreadData(&dataKey, \
-	    sizeof(ThreadSpecificData))
+#define TSD_INIT() ThreadSpecificData *tsdPtr = \
+	Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData))
 
 static void TkMacOSXNotifyExitHandler(ClientData clientData);
 static void TkMacOSXEventsSetupProc(ClientData clientData, int flags);
@@ -35,12 +34,12 @@ static void TkMacOSXEventsCheckProc(ClientData clientData, int flags);
 #pragma mark TKApplication(TKNotify)
 
 @interface NSApplication(TKNotify)
-/* We need to declare this hidden method. */
-- (void)_modalSession:(NSModalSession)session sendEvent:(NSEvent *)event;
+- (void) _modalSession: (NSModalSession) session sendEvent: (NSEvent *) event;
 @end
 
 @implementation NSWindow(TKNotify)
-- (id)tkDisplayIfNeeded {
+- (id) tkDisplayIfNeeded
+{
     if (![self isAutodisplay]) {
 	[self displayIfNeeded];
     }
@@ -49,25 +48,41 @@ static void TkMacOSXEventsCheckProc(ClientData clientData, int flags);
 @end
 
 @implementation TKApplication(TKNotify)
-/* Display all windows each time an event is removed from the queue.*/
-- (NSEvent *)nextEventMatchingMask:(NSUInteger)mask
-	untilDate:(NSDate *)expiration inMode:(NSString *)mode
-	dequeue:(BOOL)deqFlag {
-    NSEvent *event = [super nextEventMatchingMask:mask
-					untilDate:expiration
-					   inMode:mode
-					  dequeue:deqFlag];
-    /* Retain this event for later use. Must be released.*/
-    [event retain];
+- (NSEvent *) nextEventMatchingMask: (NSUInteger) mask
+	untilDate: (NSDate *) expiration inMode: (NSString *) mode
+	dequeue: (BOOL) deqFlag
+{
+    NSAutoreleasePool *pool = [NSAutoreleasePool new];
+
     [NSApp makeWindowsPerform:@selector(tkDisplayIfNeeded) inOrder:NO];
-    return event;
+
+    int oldMode = Tcl_SetServiceMode(TCL_SERVICE_ALL);
+    NSEvent *event = [[super nextEventMatchingMask:mask untilDate:expiration
+	    inMode:mode dequeue:deqFlag] retain];
+
+    Tcl_SetServiceMode(oldMode);
+    if (event) {
+	TSD_INIT();
+	if (tsdPtr->sendEventNestingLevel) {
+	    if (![NSApp tkProcessEvent:event]) {
+		[event release];
+		event = nil;
+	    }
+	}
+    }
+    [pool drain];
+    return [event autorelease];
 }
 
- /*
- * Call super then check the pasteboard.
-  */
-- (void)sendEvent:(NSEvent *)theEvent {
+- (void) sendEvent: (NSEvent *) theEvent
+{
+    TSD_INIT();
+    int oldMode = Tcl_SetServiceMode(TCL_SERVICE_ALL);
+
+    tsdPtr->sendEventNestingLevel++;
     [super sendEvent:theEvent];
+    tsdPtr->sendEventNestingLevel--;
+    Tcl_SetServiceMode(oldMode);
     [NSApp tkCheckPasteboard];
 }
 @end
@@ -128,6 +143,7 @@ void
 Tk_MacOSXSetupTkNotifier(void)
 {
     TSD_INIT();
+
     if (!tsdPtr->initialized) {
 	tsdPtr->initialized = 1;
 
@@ -145,8 +161,7 @@ Tk_MacOSXSetupTkNotifier(void)
 		    "first [load] of TkAqua has to occur in the main thread!");
 	    }
 	    Tcl_CreateEventSource(TkMacOSXEventsSetupProc,
-				  TkMacOSXEventsCheckProc,
-				  GetMainEventQueue());
+		    TkMacOSXEventsCheckProc, GetMainEventQueue());
 	    TkCreateExitHandler(TkMacOSXNotifyExitHandler, NULL);
 	    Tcl_SetServiceMode(TCL_SERVICE_ALL);
 	    TclMacOSXNotifierAddRunLoopMode(NSEventTrackingRunLoopMode);
@@ -177,9 +192,9 @@ TkMacOSXNotifyExitHandler(
     ClientData clientData)	/* Not used. */
 {
     TSD_INIT();
+
     Tcl_DeleteEventSource(TkMacOSXEventsSetupProc,
-			  TkMacOSXEventsCheckProc,
-			  GetMainEventQueue());
+	    TkMacOSXEventsCheckProc, GetMainEventQueue());
     tsdPtr->initialized = 0;
 }
 
@@ -188,19 +203,16 @@ TkMacOSXNotifyExitHandler(
  *
  * TkMacOSXEventsSetupProc --
  *
- *	This procedure implements the setup part of the MacOSX event
- *	source. It is invoked by Tcl_DoOneEvent before calling
- *      TkMacOSXEventsProc to process all queued NSEvents.  In our
- *      case, all we need to do is to set the Tcl MaxBlockTime to
- *      0 before starting the loop to process all queued NSEvents.
+ *	This procedure implements the setup part of the TkAqua Events event
+ *	source. It is invoked by Tcl_DoOneEvent before entering the notifier
+ *	to check for events.
  *
  * Results:
  *	None.
  *
  * Side effects:
- *
- *	If NSEvents are queued, then the maximum block time will be set
- *	to 0 to ensure that control returns immediately to Tcl.
+ *	If TkAqua events are queued, then the maximum block time will be set
+ *	to 0 to ensure that the notifier returns control to Tcl.
  *
  *----------------------------------------------------------------------
  */
@@ -210,20 +222,24 @@ TkMacOSXEventsSetupProc(
     ClientData clientData,
     int flags)
 {
-    NSString *runloopMode = [[NSRunLoop currentRunLoop] currentMode];
-    /* runloopMode will be nil if we are in the Tcl event loop. */
-    if (flags & TCL_WINDOW_EVENTS && !runloopMode) {
-	static Tcl_Time zeroBlockTime = { 0, 0 };
-	/* Call this with dequeue=NO -- just checking if the queue is empty. */
-	NSEvent *currentEvent = [NSApp nextEventMatchingMask:NSAnyEventMask
-				       untilDate:[NSDate distantPast]
-				       inMode:GetRunLoopMode(TkMacOSXGetModalSession())
-				       dequeue:NO];
-	if (currentEvent) {
-	    if (currentEvent.type > 0) {
-		Tcl_SetMaxBlockTime(&zeroBlockTime);
+    if (flags & TCL_WINDOW_EVENTS &&
+	    ![[NSRunLoop currentRunLoop] currentMode]) {
+	static const Tcl_Time zeroBlockTime = { 0, 0 };
+	TSD_INIT();
+
+	if (!tsdPtr->currentEvent) {
+	    NSEvent *currentEvent = [NSApp nextEventMatchingMask:NSAnyEventMask
+		    untilDate:[NSDate distantPast]
+		    inMode:GetRunLoopMode(TkMacOSXGetModalSession())
+		    dequeue:YES];
+
+	    if (currentEvent) {
+		tsdPtr->currentEvent =
+			TkMacOSXMakeUncollectableAndRetain(currentEvent);
 	    }
-	    [currentEvent release];
+	}
+	if (tsdPtr->currentEvent) {
+	    Tcl_SetMaxBlockTime(&zeroBlockTime);
 	}
     }
 }
@@ -233,69 +249,69 @@ TkMacOSXEventsSetupProc(
  *
  * TkMacOSXEventsCheckProc --
  *
- *	This procedure loops through all NSEvents waiting in the
- *      TKApplication event queue, generating X events from them.
+ *	This procedure processes events sitting in the TkAqua event queue.
  *
  * Results:
  *	None.
  *
  * Side effects:
- *	NSevents are used to generate X events, which are added to the
- *      Tcl event queue.
+ *	Moves applicable queued TkAqua events onto the Tcl event queue.
  *
  *----------------------------------------------------------------------
  */
+
 static void
 TkMacOSXEventsCheckProc(
     ClientData clientData,
     int flags)
 {
-    NSString *runloopMode = [[NSRunLoop currentRunLoop] currentMode];
-    /* runloopMode will be nil if we are in the Tcl event loop. */
-    if (flags & TCL_WINDOW_EVENTS && !runloopMode) {
+    if (flags & TCL_WINDOW_EVENTS &&
+	    ![[NSRunLoop currentRunLoop] currentMode]) {
 	NSEvent *currentEvent = nil;
-	NSEvent *testEvent = nil;
+	NSAutoreleasePool *pool = nil;
 	NSModalSession modalSession;
 
+	TSD_INIT();
+	if (tsdPtr->currentEvent) {
+	    currentEvent = TkMacOSXMakeCollectableAndAutorelease(
+	    	    tsdPtr->currentEvent);
+	}
 	do {
-	    [NSApp _resetAutoreleasePool];
 	    modalSession = TkMacOSXGetModalSession();
-	    testEvent = [NSApp nextEventMatchingMask:NSAnyEventMask
-					      untilDate:[NSDate distantPast]
-						 inMode:GetRunLoopMode(modalSession)
-						dequeue:NO];
-	    /* We must not steal any events during LiveResize. */
-	    if (testEvent && [[testEvent window] inLiveResize]) {
+	    if (!currentEvent) {
+		currentEvent = [NSApp nextEventMatchingMask:NSAnyEventMask
+			untilDate:[NSDate distantPast]
+			inMode:GetRunLoopMode(modalSession) dequeue:YES];
+	    }
+	    if (!currentEvent) {
 		break;
 	    }
-
-	    currentEvent = [NSApp nextEventMatchingMask:NSAnyEventMask
-					      untilDate:[NSDate distantPast]
-						 inMode:GetRunLoopMode(modalSession)
-						dequeue:YES];
+	    [currentEvent retain];
+	    pool = [NSAutoreleasePool new];
+	    if (tkMacOSXGCEnabled) {
+		objc_clear_stack(0);
+	    }
+	    if (![NSApp tkProcessEvent:currentEvent]) {
+		[currentEvent release];
+		currentEvent = nil;
+	    }
 	    if (currentEvent) {
-		/* Generate Xevents. */
-		int oldServiceMode = Tcl_SetServiceMode(TCL_SERVICE_ALL);
-		NSEvent *processedEvent = [NSApp tkProcessEvent:currentEvent];
-		Tcl_SetServiceMode(oldServiceMode);
-		if (processedEvent) { /* Should always be non-NULL. */
 #ifdef TK_MAC_DEBUG_EVENTS
-		    TKLog(@"   event: %@", currentEvent);
+		TKLog(@"   event: %@", currentEvent);
 #endif
-		    if (modalSession) {
-			[NSApp _modalSession:modalSession sendEvent:currentEvent];
-		    } else {
-			[NSApp sendEvent:currentEvent];
-		    }
+		if (modalSession) {
+		    [NSApp _modalSession:modalSession sendEvent:currentEvent];
+		} else {
+		    [NSApp sendEvent:currentEvent];
 		}
 		[currentEvent release];
-	    } else {
-		break;
+		currentEvent = nil;
 	    }
+	    [pool drain];
+	    pool = nil;
 	} while (1);
     }
 }
-
 
 /*
  * Local Variables:
