@@ -155,29 +155,6 @@ TkUnionRectWithRegion(
     }
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TkMacOSXIsEmptyRegion --
- *
- *	Return native region for given tk region.
- *
- * Results:
- *	1 if empty, 0 otherwise.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-int
-TkMacOSXIsEmptyRegion(
-    TkRegion r)
-{
-    return HIShapeIsEmpty((HIMutableShapeRef) r) ? 1 : 0;
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -204,14 +181,12 @@ TkRectInRegion(
     unsigned int width,
     unsigned int height)
 {
-    if ( TkMacOSXIsEmptyRegion(region) ) {
-	    return RectangleOut;
-	}
-    else {    
-	const CGRect r = CGRectMake(x, y, width, height);
-	return HIShapeIntersectsRect((HIShapeRef) region, &r) ?
+    int result;
+    const CGRect r = CGRectMake(x, y, width, height);
+
+    result = HIShapeIntersectsRect((HIShapeRef) region, &r) ?
 	    RectanglePart : RectangleOut;
-    }
+    return result;
 }
 
 /*
@@ -357,11 +332,12 @@ TkpReleaseRegion(
 {
     CFRelease(r);
 }
+#if 0
 
 /*
  *----------------------------------------------------------------------
  *
- * TkMacOSXSetEmptyRegion --
+ * TkMacOSXEmtpyRegion --
  *
  *	Set region to emtpy.
  *
@@ -375,12 +351,36 @@ TkpReleaseRegion(
  */
 
 void
-TkMacOSXSetEmptyRegion(
+TkMacOSXEmtpyRegion(
     TkRegion r)
 {
     ChkErr(HIShapeSetEmpty, (HIMutableShapeRef) r);
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkMacOSXIsEmptyRegion --
+ *
+ *	Return native region for given tk region.
+ *
+ * Results:
+ *	1 if empty, 0 otherwise.
+ *
+ * Side effects:
+ *	None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+TkMacOSXIsEmptyRegion(
+    TkRegion r)
+{
+    return HIShapeIsEmpty((HIMutableShapeRef) r) ? 1 : 0;
+}
+#endif
+
 /*
  *----------------------------------------------------------------------
  *
