@@ -1,22 +1,53 @@
 /*
  * tkAppInit.c --
  *
- *	Provides a default version of the Tcl_AppInit procedure for use in
- *	wish and similar Tk-based applications.
+ *	Provides a default version of the main program and Tcl_AppInit
+ *	procedure for wish and other Tk-based applications.
  *
  * Copyright (c) 1993 The Regents of the University of California.
  * Copyright (c) 1994-1997 Sun Microsystems, Inc.
+ * Copyright (c) 1998-1999 Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  */
 
+#undef BUILD_tk
+#undef STATIC_BUILD
 #include "tk.h"
-#include "locale.h"
 
 #ifdef TK_TEST
-extern int		Tktest_Init(Tcl_Interp *interp);
+extern Tcl_PackageInitProc Tktest_Init;
 #endif /* TK_TEST */
+
+/*
+ * The following #if block allows you to change the AppInit function by using
+ * a #define of TCL_LOCAL_APPINIT instead of rewriting this entire file. The
+ * #if checks for that #define and uses Tcl_AppInit if it doesn't exist.
+ */
+
+#ifndef TK_LOCAL_APPINIT
+#define TK_LOCAL_APPINIT Tcl_AppInit
+#endif
+#ifndef MODULE_SCOPE
+#   define MODULE_SCOPE extern
+#endif
+MODULE_SCOPE int TK_LOCAL_APPINIT(Tcl_Interp *);
+MODULE_SCOPE int main(int, char **);
+
+/*
+ * The following #if block allows you to change how Tcl finds the startup
+ * script, prime the library or encoding paths, fiddle with the argv, etc.,
+ * without needing to rewrite Tk_Main()
+ */
+
+#ifdef TK_LOCAL_MAIN_HOOK
+MODULE_SCOPE int TK_LOCAL_MAIN_HOOK(int *argc, char ***argv);
+#endif
+
+/* Make sure the stubbed variants of those are never used. */
+#undef Tcl_ObjSetVar2
+#undef Tcl_NewStringObj
 
 /*
  *----------------------------------------------------------------------
@@ -30,7 +61,7 @@ extern int		Tktest_Init(Tcl_Interp *interp);
  *	either.
  *
  * Side effects:
- *	Whatever the application does.
+ *	Just about anything, since from here we call arbitrary Tcl code.
  *
  *----------------------------------------------------------------------
  */
@@ -40,26 +71,7 @@ main(
     int argc,			/* Number of command-line arguments. */
     char **argv)		/* Values of command-line arguments. */
 {
-    /*
-     * The following #if block allows you to change the AppInit function by
-     * using a #define of TCL_LOCAL_APPINIT instead of rewriting this entire
-     * file. The #if checks for that #define and uses Tcl_AppInit if it
-     * doesn't exist.
-     */
-
-#ifndef TK_LOCAL_APPINIT
-#define TK_LOCAL_APPINIT Tcl_AppInit
-#endif
-    extern int TK_LOCAL_APPINIT (Tcl_Interp *interp);
-
-    /*
-     * The following #if block allows you to change how Tcl finds the startup
-     * script, prime the library or encoding paths, fiddle with the argv,
-     * etc., without needing to rewrite Tk_Main()
-     */
-
 #ifdef TK_LOCAL_MAIN_HOOK
-    extern int TK_LOCAL_MAIN_HOOK (int *argc, char ***argv);
     TK_LOCAL_MAIN_HOOK(&argc, &argv);
 #endif
 
@@ -90,19 +102,20 @@ int
 Tcl_AppInit(
     Tcl_Interp *interp)		/* Interpreter for application. */
 {
-    if (Tcl_Init(interp) == TCL_ERROR) {
+    if ((Tcl_Init)(interp) == TCL_ERROR) {
 	return TCL_ERROR;
     }
+
     if (Tk_Init(interp) == TCL_ERROR) {
 	return TCL_ERROR;
     }
     Tcl_StaticPackage(interp, "Tk", Tk_Init, Tk_SafeInit);
+
 #ifdef TK_TEST
     if (Tktest_Init(interp) == TCL_ERROR) {
 	return TCL_ERROR;
     }
-    Tcl_StaticPackage(interp, "Tktest", Tktest_Init,
-            (Tcl_PackageInitProc *) NULL);
+    Tcl_StaticPackage(interp, "Tktest", Tktest_Init, 0);
 #endif /* TK_TEST */
 
     /*
@@ -113,7 +126,8 @@ Tcl_AppInit(
      *     return TCL_ERROR;
      * }
      *
-     * where "Mod" is the name of the module.
+     * where "Mod" is the name of the module. (Dynamically-loadable packages
+     * should have the same entry-point name.)
      */
 
     /*
@@ -125,10 +139,11 @@ Tcl_AppInit(
      * Specify a user-specific startup file to invoke if the application is
      * run interactively. Typically the startup file is "~/.apprc" where "app"
      * is the name of the application. If this line is deleted then no user-
-     * -specific startup file will be run under any conditions.
+     * specific startup file will be run under any conditions.
      */
 
-    Tcl_SetVar(interp, "tcl_rcFileName", "~/.wishrc", TCL_GLOBAL_ONLY);
+    Tcl_ObjSetVar2(interp, Tcl_NewStringObj("tcl_rcFileName", -1), NULL,
+	    Tcl_NewStringObj("~/.wishrc", -1), TCL_GLOBAL_ONLY);
     return TCL_OK;
 }
 
