@@ -46,7 +46,8 @@ typedef struct {
 TkBorder *
 TkpGetBorder(void)
 {
-    UnixBorder *borderPtr = (UnixBorder *) ckalloc(sizeof(UnixBorder));
+    UnixBorder *borderPtr = ckalloc(sizeof(UnixBorder));
+
     borderPtr->solidGC = None;
     return (TkBorder *) borderPtr;
 }
@@ -377,7 +378,7 @@ TkpGetShadows(
 	 */
 
 	/*
-	 * Compute the dark shadow color
+	 * Compute the dark shadow color.
 	 */
 
 	r = (int) borderPtr->bgColorPtr->red;
@@ -395,7 +396,7 @@ TkpGetShadows(
 	}
 
 	/*
-	 * Allocate the dark shadow color and its GC
+	 * Allocate the dark shadow color and its GC.
 	 */
 
 	borderPtr->darkColorPtr = Tk_GetColorByValue(tkwin, &darkColor);
@@ -403,7 +404,7 @@ TkpGetShadows(
 	borderPtr->darkGC = Tk_GetGC(tkwin, GCForeground, &gcValues);
 
 	/*
-	 * Compute the light shadow color
+	 * Compute the light shadow color.
 	 */
 
 	if (g > MAX_INTENSITY*0.95) {
@@ -431,9 +432,9 @@ TkpGetShadows(
 	    lightColor.blue = (tmp1 > tmp2) ? tmp1 : tmp2;
 	}
 
-       /*
-        * Allocate the light shadow color and its GC
-        */
+	/*
+	 * Allocate the light shadow color and its GC.
+	 */
 
 	borderPtr->lightColorPtr = Tk_GetColorByValue(tkwin, &lightColor);
 	gcValues.foreground = borderPtr->lightColorPtr->pixel;
