@@ -68,7 +68,7 @@
 #define DEF_BUTTON_HIGHLIGHT		HIGHLIGHT
 #define DEF_LABEL_HIGHLIGHT_WIDTH	"0"
 #define DEF_BUTTON_HIGHLIGHT_WIDTH	"1"
-#define DEF_BUTTON_IMAGE		(char *) NULL
+#define DEF_BUTTON_IMAGE		((char *) NULL)
 #define DEF_BUTTON_INDICATOR		"1"
 #define DEF_BUTTON_JUSTIFY		"center"
 #define DEF_BUTTON_OFF_VALUE		"0"
@@ -84,10 +84,10 @@
 #define DEF_BUTTON_REPEAT_INTERVAL	"0"
 #define DEF_BUTTON_SELECT_COLOR		INDICATOR
 #define DEF_BUTTON_SELECT_MONO		BLACK
-#define DEF_BUTTON_SELECT_IMAGE		(char *) NULL
+#define DEF_BUTTON_SELECT_IMAGE		((char *) NULL)
 #define DEF_BUTTON_STATE		"normal"
 #define DEF_LABEL_TAKE_FOCUS		"0"
-#define DEF_BUTTON_TAKE_FOCUS		(char *) NULL
+#define DEF_BUTTON_TAKE_FOCUS		((char *) NULL)
 #define DEF_BUTTON_TEXT			""
 #define DEF_BUTTON_TEXT_VARIABLE	""
 #define DEF_BUTTON_TRISTATE_VALUE	""
@@ -126,7 +126,7 @@
 #define DEF_CANVAS_SELECT_BD_MONO	"0"
 #define DEF_CANVAS_SELECT_FG_COLOR	SELECT_FG
 #define DEF_CANVAS_SELECT_FG_MONO	WHITE
-#define DEF_CANVAS_TAKE_FOCUS		(char *) NULL
+#define DEF_CANVAS_TAKE_FOCUS		((char *) NULL)
 #define DEF_CANVAS_WIDTH		"10c"
 #define DEF_CANVAS_X_SCROLL_CMD		""
 #define DEF_CANVAS_X_SCROLL_INCREMENT	"0"
@@ -167,9 +167,9 @@
 #define DEF_ENTRY_SELECT_BD_MONO	"0"
 #define DEF_ENTRY_SELECT_FG_COLOR	SELECT_FG
 #define DEF_ENTRY_SELECT_FG_MONO	WHITE
-#define DEF_ENTRY_SHOW			(char *) NULL
+#define DEF_ENTRY_SHOW			((char *) NULL)
 #define DEF_ENTRY_STATE			"normal"
-#define DEF_ENTRY_TAKE_FOCUS		(char *) NULL
+#define DEF_ENTRY_TAKE_FOCUS		((char *) NULL)
 #define DEF_ENTRY_TEXT_VARIABLE		""
 #define DEF_ENTRY_WIDTH			"20"
 
@@ -235,36 +235,36 @@
 #define DEF_LISTBOX_SELECT_MODE		"browse"
 #define DEF_LISTBOX_SET_GRID		"0"
 #define DEF_LISTBOX_STATE		"normal"
-#define DEF_LISTBOX_TAKE_FOCUS		(char *) NULL
+#define DEF_LISTBOX_TAKE_FOCUS		((char *) NULL)
 #define DEF_LISTBOX_WIDTH		"20"
 
 /*
  * Defaults for individual entries of menus:
  */
 
-#define DEF_MENU_ENTRY_ACTIVE_BG	(char *) NULL
-#define DEF_MENU_ENTRY_ACTIVE_FG	(char *) NULL
-#define DEF_MENU_ENTRY_ACCELERATOR	(char *) NULL
-#define DEF_MENU_ENTRY_BG		(char *) NULL
+#define DEF_MENU_ENTRY_ACTIVE_BG	((char *) NULL)
+#define DEF_MENU_ENTRY_ACTIVE_FG	((char *) NULL)
+#define DEF_MENU_ENTRY_ACCELERATOR	((char *) NULL)
+#define DEF_MENU_ENTRY_BG		((char *) NULL)
 #define DEF_MENU_ENTRY_BITMAP		None
 #define DEF_MENU_ENTRY_COLUMN_BREAK	"0"
-#define DEF_MENU_ENTRY_COMMAND		(char *) NULL
+#define DEF_MENU_ENTRY_COMMAND		((char *) NULL)
 #define DEF_MENU_ENTRY_COMPOUND 	"none"
-#define DEF_MENU_ENTRY_FG		(char *) NULL
-#define DEF_MENU_ENTRY_FONT		(char *) NULL
+#define DEF_MENU_ENTRY_FG		((char *) NULL)
+#define DEF_MENU_ENTRY_FONT		((char *) NULL)
 #define DEF_MENU_ENTRY_HIDE_MARGIN	"0"
-#define DEF_MENU_ENTRY_IMAGE		(char *) NULL
+#define DEF_MENU_ENTRY_IMAGE		((char *) NULL)
 #define DEF_MENU_ENTRY_INDICATOR	"1"
-#define DEF_MENU_ENTRY_LABEL		(char *) NULL
-#define DEF_MENU_ENTRY_MENU		(char *) NULL
+#define DEF_MENU_ENTRY_LABEL		((char *) NULL)
+#define DEF_MENU_ENTRY_MENU		((char *) NULL)
 #define DEF_MENU_ENTRY_OFF_VALUE	"0"
 #define DEF_MENU_ENTRY_ON_VALUE		"1"
-#define DEF_MENU_ENTRY_SELECT_IMAGE	(char *) NULL
+#define DEF_MENU_ENTRY_SELECT_IMAGE	((char *) NULL)
 #define DEF_MENU_ENTRY_STATE		"normal"
-#define DEF_MENU_ENTRY_VALUE		(char *) NULL
-#define DEF_MENU_ENTRY_CHECK_VARIABLE	(char *) NULL
+#define DEF_MENU_ENTRY_VALUE		((char *) NULL)
+#define DEF_MENU_ENTRY_CHECK_VARIABLE	((char *) NULL)
 #define DEF_MENU_ENTRY_RADIO_VARIABLE	"selectedButton"
-#define DEF_MENU_ENTRY_SELECT	(char *) NULL
+#define DEF_MENU_ENTRY_SELECT		((char *) NULL)
 #define DEF_MENU_ENTRY_UNDERLINE	"-1"
 
 /*
@@ -290,7 +290,7 @@
 #define DEF_MENU_SELECT_MONO		BLACK
 #define DEF_MENU_TAKE_FOCUS		"0"
 #define DEF_MENU_TEAROFF		"1"
-#define DEF_MENU_TEAROFF_CMD		(char *) NULL
+#define DEF_MENU_TEAROFF_CMD		((char *) NULL)
 #define DEF_MENU_TITLE			""
 #define DEF_MENU_TYPE			"normal"
 
@@ -318,7 +318,7 @@
 #define DEF_MENUBUTTON_HIGHLIGHT_BG_MONO  DEF_MENUBUTTON_BG_MONO
 #define DEF_MENUBUTTON_HIGHLIGHT	HIGHLIGHT
 #define DEF_MENUBUTTON_HIGHLIGHT_WIDTH	"0"
-#define DEF_MENUBUTTON_IMAGE		(char *) NULL
+#define DEF_MENUBUTTON_IMAGE		((char *) NULL)
 #define DEF_MENUBUTTON_INDICATOR	"0"
 #define DEF_MENUBUTTON_JUSTIFY		"center"
 #define DEF_MENUBUTTON_MENU		""
@@ -370,7 +370,6 @@
 #define DEF_PANEDWINDOW_HEIGHT		""
 #define DEF_PANEDWINDOW_OPAQUERESIZE	"1"
 #define DEF_PANEDWINDOW_ORIENT		"horizontal"
-#define DEF_PANEDWINDOW_PROXYBORDER	"2"
 #define DEF_PANEDWINDOW_RELIEF		"flat"
 #define DEF_PANEDWINDOW_SASHCURSOR	""
 #define DEF_PANEDWINDOW_SASHPAD		"0"
@@ -419,7 +418,7 @@
 #define DEF_SCALE_LENGTH		"100"
 #define DEF_SCALE_ORIENT		"vertical"
 #define DEF_SCALE_RELIEF		"flat"
-#define DEF_SCALE_REPEAT_DELAY	"300"
+#define DEF_SCALE_REPEAT_DELAY		"300"
 #define DEF_SCALE_REPEAT_INTERVAL	"100"
 #define DEF_SCALE_RESOLUTION		"1"
 #define DEF_SCALE_TROUGH_COLOR		TROUGH
@@ -428,7 +427,7 @@
 #define DEF_SCALE_SLIDER_LENGTH		"30"
 #define DEF_SCALE_SLIDER_RELIEF		"raised"
 #define DEF_SCALE_STATE			"normal"
-#define DEF_SCALE_TAKE_FOCUS		(char *) NULL
+#define DEF_SCALE_TAKE_FOCUS		((char *) NULL)
 #define DEF_SCALE_TICK_INTERVAL		"0"
 #define DEF_SCALE_TO			"100"
 #define DEF_SCALE_VARIABLE		""
@@ -455,7 +454,7 @@
 #define DEF_SCROLLBAR_RELIEF		"sunken"
 #define DEF_SCROLLBAR_REPEAT_DELAY	"300"
 #define DEF_SCROLLBAR_REPEAT_INTERVAL	"100"
-#define DEF_SCROLLBAR_TAKE_FOCUS	(char *) NULL
+#define DEF_SCROLLBAR_TAKE_FOCUS	((char *) NULL)
 #define DEF_SCROLLBAR_TROUGH_COLOR	TROUGH
 #define DEF_SCROLLBAR_TROUGH_MONO	WHITE
 #define DEF_SCROLLBAR_WIDTH		"10"
@@ -482,8 +481,9 @@
 #define DEF_TEXT_INSERT_BD_MONO		"0"
 #define DEF_TEXT_INSERT_OFF_TIME	"300"
 #define DEF_TEXT_INSERT_ON_TIME		"600"
+#define DEF_TEXT_INSERT_UNFOCUSSED	"none"
 #define DEF_TEXT_INSERT_WIDTH		"2"
-#define DEF_TEXT_MAX_UNDO    	"0"
+#define DEF_TEXT_MAX_UNDO		"0"
 #define DEF_TEXT_PADX			"1"
 #define DEF_TEXT_PADY			"1"
 #define DEF_TEXT_RELIEF			"sunken"
@@ -502,8 +502,8 @@
 #define DEF_TEXT_STATE			"normal"
 #define DEF_TEXT_TABS			""
 #define DEF_TEXT_TABSTYLE		"tabular"
-#define DEF_TEXT_TAKE_FOCUS		(char *) NULL
-#define DEF_TEXT_UNDO    		"0"
+#define DEF_TEXT_TAKE_FOCUS		((char *) NULL)
+#define DEF_TEXT_UNDO			"0"
 #define DEF_TEXT_WIDTH			"80"
 #define DEF_TEXT_WRAP			"char"
 #define DEF_TEXT_XSCROLL_COMMAND	""
@@ -525,4 +525,10 @@
 #define DEF_TOPLEVEL_SCREEN		""
 #define DEF_TOPLEVEL_USE		""
 
+/*
+ * Defaults for busy windows:
+ */
+
+#define DEF_BUSY_CURSOR			"wait"
+
 #endif /* _TKWINDEFAULT */
