@@ -69,15 +69,15 @@ void
 TkWinCleanupContainerList(void)
 {
     Container *nextPtr;
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+    ThreadSpecificData *tsdPtr =
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
-    for (; tsdPtr->firstContainerPtr != (Container *) NULL;
+    for (; tsdPtr->firstContainerPtr != NULL;
 	    tsdPtr->firstContainerPtr = nextPtr) {
 	nextPtr = tsdPtr->firstContainerPtr->nextPtr;
-	ckfree((char *) tsdPtr->firstContainerPtr);
+	ckfree(tsdPtr->firstContainerPtr);
     }
-    tsdPtr->firstContainerPtr = (Container *) NULL;
+    tsdPtr->firstContainerPtr = NULL;
 }
 
 /*
@@ -102,7 +102,7 @@ TkpTestembedCmd(
     ClientData clientData,
     Tcl_Interp *interp,
     int argc,
-    CONST char **argv)
+    const char **argv)
 {
     return TCL_OK;
 }
@@ -134,7 +134,7 @@ Tk_DetachEmbeddedWindow(
 	TkpWinToplevelOverrideRedirect(winPtr, 0);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -230,7 +230,7 @@ TkpUseWindow(
 				 * string is bogus. */
     Tk_Window tkwin,		/* Tk window that does not yet have an
 				 * associated X window. */
-    CONST char *string)		/* String identifying an X window to use for
+    const char *string)		/* String identifying an X window to use for
 				 * tkwin; must be an integer value. */
 {
     TkWindow *winPtr = (TkWindow *) tkwin;
@@ -243,8 +243,9 @@ TkpUseWindow(
 
 /*
     if (winPtr->window != None) {
-	Tcl_AppendResult(interp,
-		"can't modify container after widget is created", NULL);
+	Tcl_SetObjResult(interp, Tcl_NewStringObj(
+		"can't modify container after widget is created", -1));
+	Tcl_SetErrorCode(interp, "TK", "EMBED", "POST_CREATE", NULL);
 	return TCL_ERROR;
     }
 */
@@ -256,13 +257,10 @@ TkpUseWindow(
 	return TCL_OK;
     }
 
-    if (
-#ifdef _WIN64
-	    (sscanf(string, "0x%p", &hwnd) != 1) &&
-#endif
-	    Tcl_GetInt(interp, string, (int *) &hwnd) != TCL_OK) {
+    if (Tcl_GetInt(interp, string, &id) != TCL_OK) {
 	return TCL_ERROR;
     }
+    hwnd = (HWND) INT2PTR(id);
     if ((HWND)winPtr->privatePtr == hwnd) {
 	return TCL_OK;
     }
@@ -275,8 +273,9 @@ TkpUseWindow(
 
     if (!IsWindow(hwnd)) {
 	if (interp != NULL) {
-	    Tcl_AppendResult(interp, "window \"", string,
-		    "\" doesn't exist", NULL);
+	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		    "window \"%s\" doesn't exist", string));
+	    Tcl_SetErrorCode(interp, "TK", "EMBED", "EXIST", NULL);
 	}
 	return TCL_ERROR;
     }
@@ -284,12 +283,15 @@ TkpUseWindow(
     id = SendMessage(hwnd, TK_INFO, TK_CONTAINER_VERIFY, 0);
     if (id == PTR2INT(hwnd)) {
 	if (!SendMessage(hwnd, TK_INFO, TK_CONTAINER_ISAVAILABLE, 0)) {
-    	    Tcl_AppendResult(interp, "The container is already in use", NULL);
+    	    Tcl_SetObjResult(interp, Tcl_NewStringObj(
+		    "The container is already in use", -1));
+	    Tcl_SetErrorCode(interp, "TK", "EMBED", "IN_USE", NULL);
 	    return TCL_ERROR;
 	}
     } else if (id == -PTR2INT(hwnd)) {
-	Tcl_AppendResult(interp, "the window to use is not a Tk container",
-		NULL);
+	Tcl_SetObjResult(interp, Tcl_NewStringObj(
+		"the window to use is not a Tk container", -1));
+	Tcl_SetErrorCode(interp, "TK", "EMBED", "CONTAINER", NULL);
 	return TCL_ERROR;
     } else {
 	/*
@@ -301,9 +303,11 @@ TkpUseWindow(
 	char msg[256];
 
 	sprintf(msg, "Unable to get information of window \"%.80s\".  Attach to this\nwindow may have unpredictable results if it is not a valid container.\n\nPress Ok to proceed or Cancel to abort attaching.", string);
-	if (IDCANCEL == MessageBox(hwnd, msg, "Tk Warning",
+	if (IDCANCEL == MessageBoxA(hwnd, msg, "Tk Warning",
 		MB_OKCANCEL | MB_ICONWARNING)) {
-    	    Tcl_SetResult(interp, "Operation has been canceled", TCL_STATIC);
+    	    Tcl_SetObjResult(interp, Tcl_NewStringObj(
+		    "Operation has been canceled", -1));
+	    Tcl_SetErrorCode(interp, "TK", "EMBED", "CANCEL", NULL);
 	    return TCL_ERROR;
 	}
     }
@@ -364,7 +368,7 @@ TkpMakeContainer(
      */
 
     Tk_MakeWindowExist(tkwin);
-    containerPtr = (Container *) ckalloc(sizeof(Container));
+    containerPtr = ckalloc(sizeof(Container));
     containerPtr->parentPtr = winPtr;
     containerPtr->parentHWnd = Tk_GetHWND(Tk_WindowId(tkwin));
     containerPtr->embeddedHWnd = NULL;
@@ -938,7 +942,7 @@ Tk_GetEmbeddedHWnd(
     }
     return NULL;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1098,7 +1102,7 @@ EmbedWindowDeleted(
 	} else {
 	    prevPtr->nextPtr = containerPtr->nextPtr;
 	}
-	ckfree((char *) containerPtr);
+	ckfree(containerPtr);
     }
 }
 
