@@ -39,9 +39,9 @@ DestroyImage(
 {
     if (imagePtr) {
 	if (imagePtr->data) {
-	    ckfree((char*)imagePtr->data);
+	    ckfree(imagePtr->data);
 	}
-	ckfree((char*)imagePtr);
+	ckfree(imagePtr);
     }
     return 0;
 }
@@ -211,7 +211,7 @@ XCreateImage(
     int bitmap_pad,
     int bytes_per_line)
 {
-    XImage* imagePtr = (XImage *) ckalloc(sizeof(XImage));
+    XImage* imagePtr = ckalloc(sizeof(XImage));
     imagePtr->width = width;
     imagePtr->height = height;
     imagePtr->xoffset = offset;
@@ -301,8 +301,7 @@ XGetImageZPixmap(
     BOOL ret;
 
     if (format != ZPixmap) {
-	TkpDisplayWarning(
-		"XGetImageZPixmap: only ZPixmap types are implemented",
+	TkpDisplayWarning("Only ZPixmap types are implemented",
 		"XGetImageZPixmap Failure");
 	return NULL;
     }
@@ -350,7 +349,7 @@ XGetImageZPixmap(
     if (depth <= 8) {
 	size += sizeof(unsigned short) * (1 << depth);
     }
-    bmInfo = (BITMAPINFO *) ckalloc((unsigned)size);
+    bmInfo = ckalloc(size);
 
     bmInfo->bmiHeader.biSize		= sizeof(BITMAPINFOHEADER);
     bmInfo->bmiHeader.biWidth		= width;
@@ -368,16 +367,16 @@ XGetImageZPixmap(
 	unsigned char *p, *pend;
 
 	GetDIBits(hdcMem, hbmp, 0, height, NULL, bmInfo, DIB_PAL_COLORS);
-	data = (unsigned char *) ckalloc(bmInfo->bmiHeader.biSizeImage);
+	data = ckalloc(bmInfo->bmiHeader.biSizeImage);
 	if (!data) {
 	    /* printf("Failed to allocate data area for XImage.\n"); */
 	    ret_image = NULL;
 	    goto cleanup;
 	}
-	ret_image = XCreateImage(display, NULL, depth, ZPixmap, 0, (char *)data,
+	ret_image = XCreateImage(display, NULL, depth, ZPixmap, 0, (char *) data,
 		width, height, 32, (int) ((width + 31) >> 3) & ~1);
 	if (ret_image == NULL) {
-	    ckfree((char *)data);
+	    ckfree(data);
 	    goto cleanup;
 	}
 
@@ -387,8 +386,8 @@ XGetImageZPixmap(
 
 	if (GetDIBits(hdcMem, hbmp, 0, height, data, bmInfo,
 		DIB_PAL_COLORS) == 0) {
-	    ckfree((char *) ret_image->data);
-	    ckfree((char *) ret_image);
+	    ckfree(ret_image->data);
+	    ckfree(ret_image);
 	    ret_image = NULL;
 	    goto cleanup;
 	}
@@ -404,16 +403,16 @@ XGetImageZPixmap(
 	unsigned char *p;
 
 	GetDIBits(hdcMem, hbmp, 0, height, NULL, bmInfo, DIB_PAL_COLORS);
-	data = (unsigned char *) ckalloc(bmInfo->bmiHeader.biSizeImage);
+	data = ckalloc(bmInfo->bmiHeader.biSizeImage);
 	if (!data) {
 	    /* printf("Failed to allocate data area for XImage.\n"); */
 	    ret_image = NULL;
 	    goto cleanup;
 	}
-	ret_image = XCreateImage(display, NULL, 8, ZPixmap, 0, (char *)data,
+	ret_image = XCreateImage(display, NULL, 8, ZPixmap, 0, (char *) data,
 		width, height, 8, (int) width);
 	if (ret_image == NULL) {
-	    ckfree((char *) data);
+	    ckfree(data);
 	    goto cleanup;
 	}
 
@@ -423,8 +422,8 @@ XGetImageZPixmap(
 
 	if (GetDIBits(hdcMem, hbmp, 0, height, data, bmInfo,
 		DIB_PAL_COLORS) == 0) {
-	    ckfree((char *) ret_image->data);
-	    ckfree((char *) ret_image);
+	    ckfree(ret_image->data);
+	    ckfree(ret_image);
 	    ret_image = NULL;
 	    goto cleanup;
 	}
@@ -435,7 +434,7 @@ XGetImageZPixmap(
 	}
     } else if (depth == 16) {
 	GetDIBits(hdcMem, hbmp, 0, height, NULL, bmInfo, DIB_RGB_COLORS);
-	data = (unsigned char *) ckalloc(bmInfo->bmiHeader.biSizeImage);
+	data = ckalloc(bmInfo->bmiHeader.biSizeImage);
 	if (!data) {
 	    /* printf("Failed to allocate data area for XImage.\n"); */
 	    ret_image = NULL;
@@ -444,7 +443,7 @@ XGetImageZPixmap(
 	ret_image = XCreateImage(display, NULL, 16, ZPixmap, 0, (char *) data,
 		width, height, 16, 0 /* will be calc'ed from bitmap_pad */);
 	if (ret_image == NULL) {
-	    ckfree((char *) data);
+	    ckfree(data);
 	    goto cleanup;
 	}
 
@@ -454,14 +453,14 @@ XGetImageZPixmap(
 
 	if (GetDIBits(hdcMem, hbmp, 0, height, ret_image->data, bmInfo,
 		DIB_RGB_COLORS) == 0) {
-	    ckfree((char *) ret_image->data);
-	    ckfree((char *) ret_image);
+	    ckfree(ret_image->data);
+	    ckfree(ret_image);
 	    ret_image = NULL;
 	    goto cleanup;
 	}
     } else {
 	GetDIBits(hdcMem, hbmp, 0, height, NULL, bmInfo, DIB_RGB_COLORS);
-	data = (unsigned char *) ckalloc(width * height * 4);
+	data = ckalloc(width * height * 4);
 	if (!data) {
 	    /* printf("Failed to allocate data area for XImage.\n"); */
 	    ret_image = NULL;
@@ -470,7 +469,7 @@ XGetImageZPixmap(
 	ret_image = XCreateImage(display, NULL, 32, ZPixmap, 0, (char *) data,
 		width, height, 0, (int) width * 4);
 	if (ret_image == NULL) {
-	    ckfree((char *) data);
+	    ckfree(data);
 	    goto cleanup;
 	}
 
@@ -484,10 +483,10 @@ XGetImageZPixmap(
 	    unsigned int byte_width, h, w;
 
 	    byte_width = ((width * 3 + 3) & ~(unsigned)3);
-	    smallBitBase = (unsigned char *) ckalloc(byte_width * height);
+	    smallBitBase = ckalloc(byte_width * height);
 	    if (!smallBitBase) {
-		ckfree((char *) ret_image->data);
-		ckfree((char *) ret_image);
+		ckfree(ret_image->data);
+		ckfree(ret_image);
 		ret_image = NULL;
 		goto cleanup;
 	    }
@@ -499,9 +498,9 @@ XGetImageZPixmap(
 
 	    if (GetDIBits(hdcMem, hbmp, 0, height, smallBitData, bmInfo,
 		    DIB_RGB_COLORS) == 0) {
-		ckfree((char *) ret_image->data);
-		ckfree((char *) ret_image);
-		ckfree((char *) smallBitBase);
+		ckfree(ret_image->data);
+		ckfree(ret_image);
+		ckfree(smallBitBase);
 		ret_image = NULL;
 		goto cleanup;
 	    }
@@ -511,7 +510,7 @@ XGetImageZPixmap(
 	     */
 
 	    for (h = 0; h < height; h++) {
-		bigBitData   = (unsigned char *) (ret_image->data + h * ret_image->bytes_per_line);
+		bigBitData   = (unsigned char *) ret_image->data + h * ret_image->bytes_per_line;
 		smallBitData = smallBitBase + h * byte_width;
 
 		for (w = 0; w < width; w++) {
@@ -526,7 +525,7 @@ XGetImageZPixmap(
 	     * Free the Device contexts, and the Bitmap.
 	     */
 
-	    ckfree((char *) smallBitBase);
+	    ckfree(smallBitBase);
 	} else {
 	    /*
 	     * Get the BITMAP info directly into the Image.
@@ -534,8 +533,8 @@ XGetImageZPixmap(
 
 	    if (GetDIBits(hdcMem, hbmp, 0, height, ret_image->data, bmInfo,
 		    DIB_RGB_COLORS) == 0) {
-		ckfree((char *) ret_image->data);
-		ckfree((char *) ret_image);
+		ckfree(ret_image->data);
+		ckfree(ret_image);
 		ret_image = NULL;
 		goto cleanup;
 	    }
@@ -544,7 +543,7 @@ XGetImageZPixmap(
 
   cleanup:
     if (bmInfo) {
-	ckfree((char *) bmInfo);
+	ckfree(bmInfo);
     }
     if (hPal) {
 	SelectPalette(hdcMem, hPalPrev1, FALSE);
@@ -639,7 +638,7 @@ XGetImage(
 	imagePtr = XGetImageZPixmap(display, d, x, y,
 		width, height, plane_mask, format);
     } else {
-	char *errMsg = NULL;
+	const char *errMsg = NULL;
 	char infoBuf[sizeof(BITMAPINFO) + sizeof(RGBQUAD)];
 	BITMAPINFO *infoPtr = (BITMAPINFO*)infoBuf;
 
@@ -661,8 +660,7 @@ XGetImage(
 
 	imagePtr = XCreateImage(display, NULL, 1, XYBitmap, 0, NULL,
 		width, height, 32, 0);
-	imagePtr->data =
-		ckalloc((unsigned) imagePtr->bytes_per_line*imagePtr->height);
+	imagePtr->data = ckalloc(imagePtr->bytes_per_line * imagePtr->height);
 
 	dc = GetDC(NULL);
 
