@@ -70,8 +70,8 @@
 #include <X11/Xutil.h>
 
 #ifndef __GNUC__
-#    define strncasecmp strnicmp
-#    define strcasecmp stricmp
+#    define strncasecmp _strnicmp
+#    define strcasecmp _stricmp
 #endif
 
 #define NBBY 8
