@@ -5,12 +5,11 @@
  *
  * Copyright (c) 1995 Sun Microsystems, Inc.
  *
- * See the file "license.terms" for information on usage and redistribution
- * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ * See the file "license.terms" for information on usage and redistribution of
+ * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  */
 
 #include "tkInt.h"
-
 
 /*
  *----------------------------------------------------------------------
@@ -19,10 +18,10 @@
  *
  *	Construct a single plane pixmap from bitmap data.
  *
- *	NOTE: This procedure has the correct behavior on Windows and
- *	the Macintosh, but not on UNIX.  This is probably because the
- *	emulation for XPutImage on those platforms compensates for whatever
- *	is wrong here :-)
+ *	NOTE: This procedure has the correct behavior on Windows and the
+ *	Macintosh, but not on UNIX. This is probably because the emulation for
+ *	XPutImage on those platforms compensates for whatever is wrong here
+ *	:-)
  *
  * Results:
  *	Returns a new Pixmap.
@@ -34,12 +33,12 @@
  */
 
 Pixmap
-XCreateBitmapFromData(display, d, data, width, height)
-    Display* display;
-    Drawable d;
-    _Xconst char* data;
-    unsigned int width;
-    unsigned int height;
+XCreateBitmapFromData(
+    Display *display,
+    Drawable d,
+    _Xconst char *data,
+    unsigned int width,
+    unsigned int height)
 {
     XImage *ximage;
     GC gc;
@@ -60,3 +59,11 @@ XCreateBitmapFromData(display, d, data, width, height)
     XFreeGC(display, gc);
     return pix;
 }
+
+/*
+ * Local Variables:
+ * mode: c
+ * c-basic-offset: 4
+ * fill-column: 78
+ * End:
+ */
